module.exports = {
  rules: {
    /**
     * rules/best-practices
     */

    // treat var statements as if they were block scoped
    // https://eslint.org/docs/rules/block-scoped-var
    'block-scoped-var': 'warn',

    // enforce that class methods use "this"
    // https://eslint.org/docs/rules/class-methods-use-this
    'class-methods-use-this': 0,

    // require return statements to either always or never specify values
    // https://eslint.org/docs/rules/consistent-return
    'consistent-return': 'warn',

    // specify curly brace conventions for all control statements
    // https://eslint.org/docs/rules/curly
    curly: 0,

    // encourages use of dot notation whenever possible
    // https://eslint.org/docs/rules/dot-notation
    'dot-notation': ['warn', { allowKeywords: true }],

    // enforces consistent newlines before or after dots
    // https://eslint.org/docs/rules/dot-location
    // prettier
    'dot-location': 0,

    // require the use of === and !==
    // https://eslint.org/docs/rules/eqeqeq
    eqeqeq: ['warn', 'always', { null: 'ignore' }],

    // Require grouped accessor pairs in object literals and classes
    // https://eslint.org/docs/rules/grouped-accessor-pairs
    'grouped-accessor-pairs': 'warn',

    // make sure for-in loops have an if statement
    // https://eslint.org/docs/rules/guard-for-in
    'guard-for-in': 'warn',

    // enforce a maximum number of classes per file
    // https://eslint.org/docs/rules/max-classes-per-file
    'max-classes-per-file': ['warn', 1],

    // disallow the use of alert, confirm, and prompt
    // https://eslint.org/docs/rules/no-alert
    'no-alert': process.env.NODE_ENV === 'production' ? 'error' : 'warn',

    // disallow use of arguments.caller or arguments.callee
    // https://eslint.org/docs/rules/no-caller
    'no-caller': 'warn',

    // disallow lexical declarations in case/default clauses
    // https://eslint.org/docs/rules/no-case-declarations
    'no-case-declarations': 'warn',

    // Disallow returning value in constructor
    // https://eslint.org/docs/rules/no-constructor-return
    'no-constructor-return': 'warn',

    // disallow else after a return in an if
    // https://eslint.org/docs/rules/no-else-return
    'no-else-return': ['warn', { allowElseIf: false }],

    // disallow empty functions, except for standalone funcs/arrows
    // https://eslint.org/docs/rules/no-empty-function
    'no-empty-function': [
      'warn',
      {
        allow: ['arrowFunctions', 'functions', 'methods'],
      },
    ],

    // disallow empty destructuring patterns
    // https://eslint.org/docs/rules/no-empty-pattern
    'no-empty-pattern': 'warn',

    // disallow adding to native types
    // https://eslint.org/docs/rules/no-extend-native
    'no-extend-native': 'warn',

    // disallow unnecessary function binding
    // https://eslint.org/docs/rules/no-extra-bind
    'no-extra-bind': 'warn',

    // disallow Unnecessary Labels
    // https://eslint.org/docs/rules/no-extra-label
    'no-extra-label': 'warn',

    // disallow fallthrough of case statements
    // https://eslint.org/docs/rules/no-fallthrough
    'no-fallthrough': 'warn',

    // disallow the use of leading or trailing decimal points in numeric literals
    // https://eslint.org/docs/rules/no-floating-decimal
    // prettier
    'no-floating-decimal': 0,

    // disallow reassignments of native objects or read-only globals
    // https://eslint.org/docs/rules/no-global-assign
    'no-global-assign': ['warn', { exceptions: [] }],

    // disallow usage of __iterator__ property
    // https://eslint.org/docs/rules/no-iterator
    'no-iterator': 'warn',

    // disallow use of labels for anything other than loops and switches
    // https://eslint.org/docs/rules/no-labels
    'no-labels': ['warn', { allowLoop: false, allowSwitch: false }],

    // disallow unnecessary nested blocks
    // https://eslint.org/docs/rules/no-lone-blocks
    'no-lone-blocks': 'warn',

    // disallow creation of functions within loops
    // https://eslint.org/docs/rules/no-loop-func
    'no-loop-func': 'warn',

    // disallow use of multiple spaces
    // https://eslint.org/docs/rules/no-multi-spaces
    // prettier
    'no-multi-spaces': 0,

    // disallow use of multiline strings
    // https://eslint.org/docs/rules/no-multi-str
    'no-multi-str': 'warn',

    // disallow use of new operator when not part of the assignment or comparison
    // https://eslint.org/docs/rules/no-new
    'no-new': 'warn',

    // disallow use of new operator for Function object
    // https://eslint.org/docs/rules/no-new-func
    'no-new-func': 'warn',

    // disallows creating new instances of String, Number, and Boolean
    // https://eslint.org/docs/rules/no-new-wrappers
    'no-new-wrappers': 'warn',

    // disallow reassignment of function parameters
    // disallow parameter object manipulation except for specific exclusions
    // rule: https://eslint.org/docs/rules/no-param-reassign.html
    'no-param-reassign': 0,

    // disallow use of assignment in return statement
    // https://eslint.org/docs/rules/no-return-assign
    'no-return-assign': ['warn', 'always'],

    // disallow redundant `return await`
    // https://eslint.org/docs/rules/no-return-await
    'no-return-await': 'warn',

    // disallow use of `javascript:` urls.
    // https://eslint.org/docs/rules/no-script-url
    'no-script-url': 'warn',

    // disallow self assignment
    // https://eslint.org/docs/rules/no-self-assign
    'no-self-assign': [
      'warn',
      {
        props: true,
      },
    ],

    // disallow use of comma operator
    // https://eslint.org/docs/rules/no-sequences
    'no-sequences': 'warn',

    // restrict what can be thrown as an exception
    // https://eslint.org/docs/rules/no-throw-literal
    'no-throw-literal': 'warn',

    // disallow usage of expressions in statement position
    // https://eslint.org/docs/rules/no-unused-expressions
    'no-unused-expressions': 0,

    // disallow unused labels
    // https://eslint.org/docs/rules/no-unused-labels
    'no-unused-labels': 'warn',

    // Disallow unnecessary catch clauses
    // https://eslint.org/docs/rules/no-useless-catch
    'no-useless-catch': 'warn',

    // disallow useless string concatenation
    // https://eslint.org/docs/rules/no-useless-concat
    'no-useless-concat': 'warn',

    // disallow unnecessary string escaping
    // https://eslint.org/docs/rules/no-useless-escape
    'no-useless-escape': 'warn',

    // disallow redundant return; keywords
    // https://eslint.org/docs/rules/no-useless-return
    'no-useless-return': 'warn',

    // disallow use of void operator
    // https://eslint.org/docs/rules/no-void
    'no-void': 'warn',

    // require using Error objects as Promise rejection reasons
    // https://eslint.org/docs/rules/prefer-promise-reject-errors
    'prefer-promise-reject-errors': ['warn', { allowEmptyReject: true }],

    // https://eslint.org/docs/rules/prefer-regex-literals
    'prefer-regex-literals': [
      'warn',
      {
        disallowRedundantWrapping: true,
      },
    ],

    // require use of the second argument for parseInt()
    // https://eslint.org/docs/rules/radix
    radix: 'warn',

    // requires to declare all vars on top of their containing scope
    // https://eslint.org/docs/rules/vars-on-top
    'vars-on-top': 'warn',

    // require or disallow Yoda conditions
    // https://eslint.org/docs/rules/yoda
    yoda: 'warn',

    /**
     * rules/errors
     */

    // Disallow await inside of loops
    // https://eslint.org/docs/rules/no-await-in-loop
    'no-await-in-loop': 0,

    // disallow assignment in conditional expressions
    'no-cond-assign': 'warn',

    // disallow use of console
    'no-console': 'warn',

    // disallow use of debugger
    'no-debugger': process.env.NODE_ENV === 'production' ? 'error' : 'warn',

    // Disallow duplicate conditions in if-else-if chains
    // https://eslint.org/docs/rules/no-dupe-else-if
    'no-dupe-else-if': 'warn',

    // disallow empty statements
    'no-empty': 'warn',

    // disallow the use of empty character classes in regular expressions
    'no-empty-character-class': 'warn',

    // disallow assigning to the exception in a catch block
    'no-ex-assign': 'warn',

    // disallow double-negation boolean casts in a boolean context
    // https://eslint.org/docs/rules/no-extra-boolean-cast
    'no-extra-boolean-cast': 'warn',

    // disallow unnecessary semicolons
    // prettier
    'no-extra-semi': 0,

    // disallow overwriting functions written as function declarations
    'no-func-assign': 'warn',

    // Disallow returning values from Promise executor functions
    // https://eslint.org/docs/rules/no-promise-executor-return
    'no-promise-executor-return': 0,

    // disallow multiple spaces in a regular expression literal
    'no-regex-spaces': 'warn',

    // Disallow returning values from setters
    // https://eslint.org/docs/rules/no-setter-return
    'no-setter-return': 'warn',

    // disallow sparse arrays
    'no-sparse-arrays': 'warn',

    // Disallow template literal placeholder syntax in regular strings
    // https://eslint.org/docs/rules/no-template-curly-in-string
    'no-template-curly-in-string': 'warn',

    // disallow unreachable statements after a return, throw, continue, or break statement
    'no-unreachable': 'warn',

    // Disallow Unused Private Class Members
    // https://eslint.org/docs/rules/no-unused-private-class-members
    'no-unused-private-class-members': 'warn',

    // Disallow useless backreferences in regular expressions
    // https://eslint.org/docs/rules/no-useless-backreference
    'no-useless-backreference': 'warn',

    // disallow comparisons with the value NaN
    'use-isnan': 'warn',

    // ensure that the results of typeof are compared against a valid string
    // https://eslint.org/docs/rules/valid-typeof
    'valid-typeof': ['warn', { requireStringLiterals: true }],

    /**
     * rules/style
     */
    // enforce spacing inside array brackets
    // prettier
    'array-bracket-spacing': 0,

    // enforce spacing inside single-line blocks
    // https://eslint.org/docs/rules/block-spacing
    // prettier
    'block-spacing': 0,

    // enforce one true brace style
    // prettier
    'brace-style': 0,

    // require camel case names
    camelcase: ['warn', { properties: 'never', ignoreDestructuring: false }],

    // require trailing commas in multiline object literals
    // prettier
    'comma-dangle': 0,

    // enforce spacing before and after comma
    // prettier
    'comma-spacing': 0,

    // disallow padding inside computed properties
    // prettier
    'computed-property-spacing': 0,

    // enforce newline at the end of file, with no multiple empty lines
    // prettier
    'eol-last': 0,

    // https://eslint.org/docs/rules/function-call-argument-newline
    // prettier
    'function-call-argument-newline': 0,

    // enforce spacing between functions and their invocations
    // https://eslint.org/docs/rules/func-call-spacing
    // prettier
    'func-call-spacing': 0,

    // require function expressions to have a name
    // https://eslint.org/docs/rules/func-names
    'func-names': 0,

    // require line breaks inside function parentheses if there are line breaks between parameters
    // https://eslint.org/docs/rules/function-paren-newline
    // prettier
    'function-paren-newline': 0,

    // prettier
    indent: 0,

    // enforces spacing between keys and values in object literal properties
    // prettier
    'key-spacing': 0,

    // prettier
    'keyword-spacing': 0,

    // disallow mixed 'LF' and 'CRLF' as linebreaks
    // https://eslint.org/docs/rules/linebreak-style
    // https://github.com/airbnb/javascript/pull/1224#issuecomment-347114680
    // prettier
    'linebreak-style': 0,

    // require or disallow an empty line between class members
    // https://eslint.org/docs/rules/lines-between-class-members
    'lines-between-class-members': [
      'warn',
      'always',
      { exceptAfterSingleLine: false },
    ],

    // require or disallow newlines around directives
    // https://eslint.org/docs/rules/lines-around-directive
    'lines-around-directive': [
      'warn',
      {
        before: 'always',
        after: 'always',
      },
    ],

    // specify the maximum length of a line in your program
    // https://eslint.org/docs/rules/max-len
    // prettier
    'max-len': 0,

    'new-cap': 'warn',

    // disallow the omission of parentheses when invoking a constructor with no arguments
    // https://eslint.org/docs/rules/new-parens
    // prettier
    'new-parens': 0,

    // enforces new line after each method call in the chain to make it
    // more readable and easy to maintain
    // https://eslint.org/docs/rules/newline-per-chained-call
    // prettier
    'newline-per-chained-call': 0,

    // disallow use of the Array constructor
    'no-array-constructor': 'warn',

    // disallow use of bitwise operators
    // https://eslint.org/docs/rules/no-bitwise
    'no-bitwise': 'warn',

    // disallow use of the continue statement
    // https://eslint.org/docs/rules/no-continue
    'no-continue': 'warn',

    // disallow if as the only statement in an else block
    // https://eslint.org/docs/rules/no-lonely-if
    'no-lonely-if': 0,

    // disallow un-paren'd mixes of different operators
    // https://eslint.org/docs/rules/no-mixed-operators
    // prettier
    'no-mixed-operators': 0,

    // disallow mixed spaces and tabs for indentation
    // prettier
    'no-mixed-spaces-and-tabs': 0,

    // disallow use of chained assignment expressions
    // https://eslint.org/docs/rules/no-multi-assign
    'no-multi-assign': ['warn'],

    // disallow multiple empty lines, only one newline at the end, and no new lines at the beginning
    // https://eslint.org/docs/rules/no-multiple-empty-lines
    // prettier
    'no-multiple-empty-lines': 0,

    // disallow nested ternary expressions
    'no-nested-ternary': 'warn',

    // disallow use of unary operators, ++ and --
    // https://eslint.org/docs/rules/no-plusplus
    'no-plusplus': [0, { allowForLoopAfterthoughts: true }],

    // disallow certain syntax forms
    // https://eslint.org/docs/rules/no-restricted-syntax
    'no-restricted-syntax': [
      'warn',
      {
        selector: 'ForInStatement',
        message:
          'for..in loops iterate over the entire prototype chain, which is virtually never what you want. Use Object.{keys,values,entries}, and iterate over the resulting array.',
      },
      {
        selector: 'ForOfStatement',
        message:
          'iterators/generators require regenerator-runtime, which is too heavyweight for this guide to allow them. Separately, loops should be avoided in favor of array iterations.',
      },
      {
        selector: 'LabeledStatement',
        message:
          'Labels are a form of GOTO; using them makes code confusing and hard to maintain and understand.',
      },
      {
        selector: 'WithStatement',
        message:
          '`with` is disallowed in strict mode because it makes code impossible to predict and optimize.',
      },
    ],

    // disallow space between function identifier and application
    'no-spaced-func': 0,

    // disallow tab characters entirely
    // prettier
    'no-tabs': 0,

    // disallow trailing whitespace at the end of lines
    // prettier
    'no-trailing-spaces': 0,

    // disallow dangling underscores in identifiers
    // https://eslint.org/docs/rules/no-underscore-dangle
    'no-underscore-dangle': 0,

    // disallow the use of Boolean literals in conditional expressions
    // also, prefer `a || b` over `a ? a : b`
    // https://eslint.org/docs/rules/no-unneeded-ternary
    'no-unneeded-ternary': ['warn', { defaultAssignment: false }],

    // disallow whitespace before properties
    // https://eslint.org/docs/rules/no-whitespace-before-property
    // prettier
    'no-whitespace-before-property': 0,

    // enforce the location of single-line statements
    // https://eslint.org/docs/rules/nonblock-statement-body-position
    // prettier
    'nonblock-statement-body-position': 0,

    // require padding inside curly braces
    // prettier
    'object-curly-spacing': 0,

    // prettier
    'object-curly-newline': 0,

    // prettier
    'object-property-newline': 0,

    // allow just one var statement per function
    'one-var': ['warn', 'never'],

    // require a newline around variable declaration
    // https://eslint.org/docs/rules/one-var-declaration-per-line
    // prettier
    'one-var-declaration-per-line': 0,

    // require assignment operator shorthand where possible or prohibit it entirely
    // https://eslint.org/docs/rules/operator-assignment
    'operator-assignment': 0,

    // Requires operator at the beginning of the line in multiline statements
    // https://eslint.org/docs/rules/operator-linebreak
    // prettier
    'operator-linebreak': 0,

    // disallow padding within blocks
    // prettier
    'padded-blocks': 0,

    // Disallow the use of Math.pow in favor of the ** operator
    // https://eslint.org/docs/rules/prefer-exponentiation-operator
    'prefer-exponentiation-operator': 0,

    // Prefer use of an object spread over Object.assign
    // https://eslint.org/docs/rules/prefer-object-spread
    'prefer-object-spread': 0,

    // require quotes around object literal property names
    // https://eslint.org/docs/rules/quote-props.html
    // prettier
    'quote-props': 0,

    // specify whether double or single quotes should be used
    // prettier
    quotes: 0,

    // require or disallow use of semicolons instead of ASI
    // prettier
    semi: 0,

    // enforce spacing before and after semicolons
    // prettier
    'semi-spacing': 0,

    // Enforce location of semicolons
    // https://eslint.org/docs/rules/semi-style
    // prettier
    'semi-style': 0,

    // require or disallow space before blocks
    // prettier
    'space-before-blocks': 0,

    // require or disallow space before function opening parenthesis
    // https://eslint.org/docs/rules/space-before-function-paren
    // prettier
    'space-before-function-paren': 0,

    // require or disallow spaces inside parentheses
    // prettier
    'space-in-parens': 0,

    // require spaces around operators
    // prettier
    'space-infix-ops': 0,

    // Require or disallow spaces before/after unary operators
    // https://eslint.org/docs/rules/space-unary-ops
    // prettier
    'space-unary-ops': 0,

    // require or disallow a space immediately following the // or /* in a comment
    // https://eslint.org/docs/rules/spaced-comment
    'spaced-comment': [
      'warn',
      'always',
      {
        line: {
          exceptions: ['-', '+'],
          markers: ['=', '!', '/'], // space here to support sprockets directives, slash for TS /// comments
        },
        block: {
          exceptions: ['-', '+'],
          markers: ['=', '!', ':', '::'], // space here to support sprockets directives and flow comment types
          balanced: true,
        },
      },
    ],

    // Enforce spacing around colons of switch statements
    // https://eslint.org/docs/rules/switch-colon-spacing
    // prettier
    'switch-colon-spacing': 0,

    // Require or disallow spacing between template tags and their literals
    // https://eslint.org/docs/rules/template-tag-spacing
    // prettier
    'template-tag-spacing': 0,

    /**
     * rules/variables
     */

    // disallow specific globals
    'no-restricted-globals': 0,

    // disallow declaration of variables already declared in the outer scope
    'no-shadow': 'warn',

    // disallow use of undefined when initializing variables
    'no-undef-init': 'warn',

    // disallow declaration of variables that are not used in the code
    'no-unused-vars': 0,

    // disallow use of variables before they are defined
    'no-use-before-define': 0,

    /**
     * rules/es6
     */

    // enforces no braces where they can be omitted
    // https://eslint.org/docs/rules/arrow-body-style
    // TODO: enable requireReturnForObjectLiteral?
    'arrow-body-style': 0,

    // require parens in arrow function arguments
    // https://eslint.org/docs/rules/arrow-parens
    // prettier
    'arrow-parens': 0,

    // require space before/after arrow function's arrow
    // https://eslint.org/docs/rules/arrow-spacing
    // prettier
    'arrow-spacing': 0,

    // enforce the spacing around the * in generator functions
    // https://eslint.org/docs/rules/generator-star-spacing
    // prettier
    'generator-star-spacing': 0,

    // disallow arrow functions where they could be confused with comparisons
    // https://eslint.org/docs/rules/no-confusing-arrow
    // prettier
    'no-confusing-arrow': 0,

    // disallow useless computed property keys
    // https://eslint.org/docs/rules/no-useless-computed-key
    'no-useless-computed-key': 0,

    // disallow unnecessary constructor
    // https://eslint.org/docs/rules/no-useless-constructor
    'no-useless-constructor': 'warn',

    // require let or const instead of var
    'no-var': 'warn',

    // require method and property shorthand syntax for object literals
    // https://eslint.org/docs/rules/object-shorthand
    'object-shorthand': [
      'warn',
      'always',
      {
        ignoreConstructors: false,
        avoidQuotes: true,
      },
    ],

    // suggest using arrow functions as callbacks
    'prefer-arrow-callback': [
      'warn',
      {
        allowNamedFunctions: false,
        allowUnboundThis: true,
      },
    ],

    // suggest using of const declaration for variables that are never modified after declared
    'prefer-const': [
      'warn',
      {
        destructuring: 'any',
        ignoreReadBeforeAssign: true,
      },
    ],

    // Prefer destructuring from arrays and objects
    // https://eslint.org/docs/rules/prefer-destructuring
    'prefer-destructuring': [
      'off',
      {
        VariableDeclarator: {
          array: false,
          object: true,
        },
        AssignmentExpression: {
          array: true,
          object: false,
        },
      },
      {
        enforceForRenamedProperties: false,
      },
    ],

    // disallow parseInt() in favor of binary, octal, and hexadecimal literals
    // https://eslint.org/docs/rules/prefer-numeric-literals
    'prefer-numeric-literals': 'warn',

    // suggest using the spread syntax instead of .apply()
    // https://eslint.org/docs/rules/prefer-spread
    'prefer-spread': 'warn',

    // suggest using template literals instead of string concatenation
    // https://eslint.org/docs/rules/prefer-template
    'prefer-template': 0,

    // disallow generator functions that do not have yield
    // https://eslint.org/docs/rules/require-yield
    'require-yield': 'warn',

    // enforce spacing between object rest-spread
    // https://eslint.org/docs/rules/rest-spread-spacing
    // prettier
    'rest-spread-spacing': 0,

    // enforce usage of spacing in template strings
    // https://eslint.org/docs/rules/template-curly-spacing
    // prettier
    'template-curly-spacing': 0,

    // enforce spacing around the * in yield* expressions
    // https://eslint.org/docs/rules/yield-star-spacing
    // prettier
    'yield-star-spacing': 0,

    /**
     * rules/imports
     */
    // disallow AMD require/define
    // https://github.com/benmosher/eslint-plugin-import/blob/master/docs/rules/no-amd.md
    'import/no-amd': 'warn',

    // disallow non-import statements appearing before import statements
    // https://github.com/benmosher/eslint-plugin-import/blob/master/docs/rules/first.md
    'import/first': 'warn',

    // Ensure consistent use of file extension within the import path
    // https://github.com/benmosher/eslint-plugin-import/blob/master/docs/rules/extensions.md
    'import/extensions': 0,

    // ensure absolute imports are above relative imports and that unassigned imports are ignored
    // https://github.com/benmosher/eslint-plugin-import/blob/master/docs/rules/order.md
    // TODO: enforce a stricter convention in module import order?
    'import/order': ['warn', { groups: [['builtin', 'external', 'internal']] }],

    // Require a newline after the last import/require in a group
    // https://github.com/benmosher/eslint-plugin-import/blob/master/docs/rules/newline-after-import.md
    'import/newline-after-import': 'warn',

    // Require modules with a single export to use a default export
    // https://github.com/benmosher/eslint-plugin-import/blob/master/docs/rules/prefer-default-export.md
    'import/prefer-default-export': 0,

    // Forbid require() calls with expressions
    // https://github.com/benmosher/eslint-plugin-import/blob/master/docs/rules/no-dynamic-require.md
    'import/no-dynamic-require': 0,

    // warn on accessing default export property names that are also named exports
    // https://github.com/benmosher/eslint-plugin-import/blob/master/docs/rules/no-named-as-default-member.md
    'import/no-named-as-default-member': 'warn',
  },
};
